/// <reference types="node" />
import { EventEmitter } from "events";
import { epicStorage } from "epic-storage";
import { Server, ServerOptions } from "socket.io";
export declare type CLIENT = {
    id: string;
    reference: string;
    name: string | null;
    contact: string | null;
    address: string | null;
    trace: any;
    meta: any;
};
export declare type CLIENTS = {
    [socketId: string]: CLIENT;
};
export declare type AUTHENTICATION_OBJECT = {
    reference: string;
    name: string | null;
    contact: string | null;
    trace: any;
    meta: any;
};
export declare type MESSAGE = {
    message: string;
    attachments: object[];
};
export declare type SENT_MESSAGE = {
    sender: CLIENT;
    message: MESSAGE | any;
};
export declare type RECEIVED_MESSAGE = {
    receiver: CLIENT;
    message: MESSAGE;
};
export declare type CONNECTION = {
    connection: CLIENT;
    connections: CLIENTS;
    connectionCount: number;
};
export declare enum EVENT_TYPE {
    UNICAST = 0,
    BROADCAST = 1,
    CAST = 2
}
export declare type EVENT = {
    type: EVENT_TYPE;
    sender?: CLIENT;
    receiver?: CLIENT;
    payload: any;
};
export declare type SSL_SECURE = {
    key: string;
    crt: string;
    ca?: string;
};
export declare class epicChatHelper {
    socket: Server | undefined;
    host: string;
    port: string | number | false;
    options: ServerOptions | undefined;
    connections: CLIENTS;
    connectionCount: number;
    events: EventEmitter;
    logStorage: epicStorage;
    logSchema: string;
    conversationStorage: epicStorage;
    contacts: string[][];
    eventPrefix: string;
    constructor(host: string, port: number | string, options?: ServerOptions);
    init: (SSL_SECURE?: SSL_SECURE | undefined) => this;
    on: (event: string, handler: any) => this;
    objectCount: (object: object) => number;
}
//# sourceMappingURL=epicChatHelper.d.ts.map