"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const http_1 = __importDefault(require("http"));
const https_1 = __importDefault(require("https"));
const fs_1 = __importDefault(require("fs"));
const epic_storage_1 = require("epic-storage");
const socket_io_1 = __importDefault(require("socket.io"));
const moment_1 = __importDefault(require("moment"));
var EVENT_TYPE;
(function (EVENT_TYPE) {
    EVENT_TYPE[EVENT_TYPE["UNICAST"] = 0] = "UNICAST";
    EVENT_TYPE[EVENT_TYPE["BROADCAST"] = 1] = "BROADCAST";
    EVENT_TYPE[EVENT_TYPE["CAST"] = 2] = "CAST";
})(EVENT_TYPE = exports.EVENT_TYPE || (exports.EVENT_TYPE = {}));
;
class epicChatHelper {
    //Constructor
    constructor(host, port, options) {
        this.connections = {};
        this.connectionCount = 0;
        this.events = new events_1.EventEmitter;
        this.logStorage = new epic_storage_1.epicStorage({
            storageName: "epicChatHelper_ActivityLogs"
        });
        this.logSchema = moment_1.default().format('YYYY-MM-DD');
        this.conversationStorage = new epic_storage_1.epicStorage({
            storageName: "epicChatHelper_Conversation"
        });
        this.contacts = [];
        this.eventPrefix = "epicChatHelper.";
        //Methods
        this.init = (SSL_SECURE) => {
            // Setup Server
            let server;
            if (SSL_SECURE)
                server = https_1.default.createServer({
                    cert: fs_1.default.readFileSync(SSL_SECURE.crt),
                    key: fs_1.default.readFileSync(SSL_SECURE.key),
                    ca: SSL_SECURE.ca ? fs_1.default.readFileSync(SSL_SECURE.ca) : undefined,
                });
            else
                server = http_1.default.createServer();
            // Start Server
            server.listen(this.port);
            this.socket = socket_io_1.default(server, this.options);
            // Listeninig For New Connections
            this.socket.on("connection", (socket) => {
                // New Connection Authenticated
                socket.on(this.eventPrefix + "authenticate", ({ reference, name, contact, trace, meta }) => {
                    var _a;
                    let authorize = false;
                    if (typeof this.connections[socket.id] != "undefined")
                        authorize = true;
                    if (typeof this.connections[socket.id] == "undefined" && name)
                        authorize = true;
                    this.connections[socket.id] = {
                        id: socket.id,
                        reference: reference,
                        name: name,
                        contact: contact,
                        address: socket.handshake.address,
                        trace: trace,
                        meta: meta
                    };
                    if (!authorize) {
                        // Update Connection Count
                        this.connectionCount = this.objectCount(this.connections);
                    }
                    else {
                        // Update Connection Count
                        this.connectionCount = this.objectCount(this.connections);
                        // Emit Event
                        let connection = {
                            connection: this.connections[socket.id],
                            connections: this.connections,
                            connectionCount: this.connectionCount,
                        };
                        (_a = this.socket) === null || _a === void 0 ? void 0 : _a.emit(this.eventPrefix + "connected", connection);
                        // Local Event
                        this.events.emit(this.eventPrefix + "connected", connection);
                        console.log("EpicChatHelper: A New Connection Authenticated.", { reference, name });
                        // Log Connection
                        (() => __awaiter(this, void 0, void 0, function* () {
                            let data;
                            try {
                                data = yield this.logStorage.useSchema(this.logSchema).getItem("connections");
                            }
                            catch (e) {
                                data = {};
                            }
                            data[socket.id] = this.connections[socket.id];
                            this.logStorage.useSchema(this.logSchema).addItem("connections", data);
                        }))();
                    }
                });
                // Send Event
                socket.on(this.eventPrefix + "send_event", (data) => {
                    var _a, _b;
                    if (data.type == EVENT_TYPE.BROADCAST)
                        socket.broadcast.emit(this.eventPrefix + "event", data);
                    else if (data.type == EVENT_TYPE.UNICAST && data.receiver)
                        (_a = this.socket) === null || _a === void 0 ? void 0 : _a.to(data.receiver.id).emit(this.eventPrefix + "event", data);
                    else
                        (_b = this.socket) === null || _b === void 0 ? void 0 : _b.emit(this.eventPrefix + "event", data);
                    this.events.emit(this.eventPrefix + "send_event", data);
                });
                // Message
                socket.on(this.eventPrefix + "message", (data) => {
                    var _a;
                    let message = {
                        sender: this.connections[socket.id],
                        message: data.message,
                    };
                    let found;
                    this.contacts.forEach((v) => {
                        if ((v[0] == socket.id && v[1] == data.receiver.id) || (v[1] == socket.id && v[0] == data.receiver.id)) {
                            found = v;
                        }
                    });
                    if (!found) {
                        this.contacts.push([socket.id, data.receiver.id]);
                        found = [socket.id, data.receiver.id];
                    }
                    (() => __awaiter(this, void 0, void 0, function* () {
                        let contacts = found[0] + "_" + found[1];
                        try {
                            yield this.conversationStorage.addSchema(contacts);
                            this.conversationStorage.useSchema(contacts).addItem(moment_1.default().format('YYYY-MM-DD HH:mm:ss') + " #" + Math.random().toString(36).substr(2, 9), message);
                        }
                        catch (e) {
                            console.log("Error Occured!", e);
                        }
                    }))();
                    (_a = this.socket) === null || _a === void 0 ? void 0 : _a.to(data.receiver.id).emit(this.eventPrefix + "message", message);
                    this.events.emit(this.eventPrefix + "message", {
                        sender: this.connections[socket.id],
                        receiver: data.receiver,
                        message: data.message,
                    });
                });
                // Connection Disconnected
                socket.on('disconnect', () => {
                    var _a;
                    let disconnected = this.connections[socket.id];
                    delete this.connections[socket.id];
                    let connection = {
                        connection: disconnected,
                        connections: this.connections,
                        connectionCount: this.connectionCount,
                    };
                    (_a = this.socket) === null || _a === void 0 ? void 0 : _a.emit(this.eventPrefix + "disconnected", connection);
                    this.events.emit(this.eventPrefix + "disconnected", connection);
                    //Log
                    console.log("EpicChatHelper: A Connection Has Been Disconnected!");
                });
            });
            return this;
        };
        this.on = (event, handler) => {
            this.events.on(this.eventPrefix + event, handler);
            return this;
        };
        //Utilities
        this.objectCount = (object) => {
            let size = 0;
            for (let key in object) {
                if (object.hasOwnProperty(key))
                    size++;
            }
            return size;
        };
        this.host = host;
        this.conversationStorage.init();
        this.logStorage.init().then(() => {
            this.logStorage.addSchema(this.logSchema);
        });
        // Normalize a port into a number, string, or false.
        let normalizePort = (port) => {
            if (typeof port == "string")
                port = parseInt(port, 10);
            if (isNaN(port)) {
                // Named pipe
                return port;
            }
            if (port >= 0) {
                // Port number
                return port;
            }
            return false;
        };
        this.port = normalizePort(port);
        this.options = options;
        return this;
    }
}
exports.epicChatHelper = epicChatHelper;
;
//# sourceMappingURL=epicChatHelper.js.map